if( chrome.extension.getBackgroundPage() == window ){
	
	const CAPTURE_WIDTH = 1024;
	const CAPTURE_HEIGHT = 768;
	const CAPTURE_TIMEOUT = 60000 * 2; // 2 minutes
	const CHECK_COMPLETE_INTERVAL = 1000;
	const CHECK_COMPLETE_INTERVAL_FINAL = 3000;
	const DISPLAY_HIDDEN_WINDOW_MONITOR = 5;
	
	function RemoveWindowListener( captureWindowId ){
		
		var self = this;
		
		function _removeWindowListener( removedWindowId ){
			
			if( removedWindowId == captureWindowId ){
				return self.stop();
			}
			
			chrome.windows.getAll( function( windows ){
				
				if( windows.length == 1 && windows[0].id == captureWindowId ){
					chrome.windows.remove( captureWindowId );
				}
				
			} );
			
		}
		
		chrome.windows.onRemoved.addListener( _removeWindowListener );
		
		this.stop = function(){
			chrome.windows.onRemoved.removeListener( _removeWindowListener );
		};
		
	}
	
	fvdSpeedDial.HiddenCapture = new function(){
		
		var isLinux = navigator.platform.toLowerCase().indexOf( "linux" ) !== -1;
		var isMac = navigator.platform.toLowerCase().indexOf( "mac" ) === 0;
		var isWin = navigator.platform.toLowerCase().indexOf( "win" ) === 0; 
				
		this.capture = function( params, callback ){
			
			if( typeof params == "string" ){
				params = {
					url: params
				};
			}
			
			
			params.width = params.width || fvdSpeedDial.SpeedDial.getMaxCellWidth();
			
			chrome.windows.create({
				url: params.url,
				focused: false,
				
				left: 100000,
				top: 100000,
				
				
				width: isWin ? 100 : 1,
				height: isWin ? 100 : 1,
								
				type: "popup"
			}, function( w ){
				
				var removeListener = new RemoveWindowListener( w.id );
				
				if( !w.tabs || w.tabs.length == 0 ){
					// close window
					chrome.windows.remove( w.id );
					return;
				}
				
				var ctimeout = null;				
				var timeout = setTimeout(function(){
					
					try{
						clearTimeout( ctimeout );						
					}
					catch( ex ){
						
					}
					
					chrome.windows.remove( w.id );
					callback( null );
															
				}, CAPTURE_TIMEOUT);
				
				
				var monitor = 0;
				fvdSpeedDial.Utils.Async.cc(function( ccCallback ){
			        chrome.windows.update(w.id, {
			        	top: 100000,
			        	left: 100000			            
			        }, function(){
						
						monitor++;
						if( monitor == DISPLAY_HIDDEN_WINDOW_MONITOR ){
							// restore size
							chrome.windows.update( w.id, {
								width: CAPTURE_WIDTH,
								height: CAPTURE_HEIGHT
							} );
							return;
						}
		
						ccCallback();
						
					});	
				});
                
				
				if( isLinux || isMac ){
			        chrome.windows.update(w.id, {
			        	state: "minimized"
			        });
				}
				
				var tab = w.tabs[0];
				
				chrome.tabs.executeScript( tab.id, {
					file: "/js/capture/contentscript.js",
					runAt: "document_start"
				});
				
				var isFinalTimeout = false;				
				function checkTimeout( interval ){
					
					ctimeout = setTimeout(function(){
						
						chrome.tabs.get( tab.id, function( tabInfo ){
							
							if( !tabInfo ){
								// tab closed
								clearTimeout( timeout );
								return callback( null );
							}
							
							if( !params.saveImage && tabInfo.title ){
								// capture only title
								capture( tabInfo );	
								return;
							}
							
							if( tabInfo.status == "complete" ){	
								if( isFinalTimeout ){
									capture( tabInfo );	
								}				
								else{
									isFinalTimeout = true;
									checkTimeout( CHECK_COMPLETE_INTERVAL_FINAL );
								}	
							}
							else{
								isFinalTimeout = false;
								checkTimeout();
							}
							
						} );
						
					}, interval || CHECK_COMPLETE_INTERVAL);
					
				}
				
				function normailzeWithCheck( callback, attemptNum ){
					
					if( attemptNum == 10 ){
					//	alert(_("dlg_alert_release_left_button"));
					}
					
					if( attemptNum > 30 ){
						return callback();
					}
					
					attemptNum = attemptNum || 0;
					
					chrome.windows.update(w.id, {
			        	state: "normal",
						focused: false
			        }, function(){
						
						chrome.windows.get( w.id, function( wInfo ){
							if( wInfo.state != "normal" ){
								normailzeWithCheck( callback, attemptNum + 1 );
							}
							else{
								callback();
							}
						} );
						
					});
						
				}
				
				function capture( tab ){
					
					fvdSpeedDial.Utils.Async.chain([
						
						function( chainCallback ){
							
							chrome.windows.get( w.id, function( wInfo ){							    
								if( wInfo.state != "normal" ){
									normailzeWithCheck( chainCallback );
								}
								else{
									chainCallback();
								}
							} );									
							
						},
						
						function(){
							
							chrome.windows.update( w.id, {
								width: CAPTURE_WIDTH,
								height: CAPTURE_HEIGHT								
							}, function(){
    							
								setTimeout(function(){
									
									chrome.tabs.captureVisibleTab( w.id, function( dataUrl ){									
										clearTimeout( timeout );	
										chrome.windows.remove( w.id );
										
										fvdSpeedDial.ThumbMaker.getImageDataPath( dataUrl, params.width, function( thumbUrl, thumbSize ){
											callback( {
												dataUrl: thumbUrl,
												title: tab.title,
												thumbSize: thumbSize
											} );										
										} );
									} );
									
								}, 500);
								
							} );
							
						}
						
					]);
					
				}
				
				checkTimeout();
				
				/*			
				function listener( tabId, changeInfo ){
					
					if( tabId == tab.id ){
						
						if( changeInfo.status && changeInfo.status == "complete" ){
																						
							chrome.tabs.onUpdated.removeListener( listener );
							
							chrome.windows.update( w.id, {
								width: CAPTURE_WIDTH,
								height: CAPTURE_HEIGHT
							}, function(){
																											
								chrome.tabs.captureVisibleTab( w.id, function( dataUrl ){									
									clearTimeout( timeout );	
									chrome.windows.remove( w.id );
									
									fvdSpeedDial.ThumbMaker.getImageDataPath( dataUrl, params.width, function( thumbUrl, thumbSize ){
										callback( {
											dataUrl: thumbUrl,
											title: tab.title,
											thumbSize: thumbSize
										} );										
									} );
								} );
								
							} );
							
						}
						
					}	
					
				}
						
				chrome.tabs.onUpdated.addListener( listener );
				*/
				
			});			
			
		};
		
	};
	
}
else{
	
	fvdSpeedDial.HiddenCapture = chrome.extension.getBackgroundPage().fvdSpeedDial.HiddenCapture;
	
}
