if( chrome.extension.getBackgroundPage() == window ){
	
	fvdSpeedDial.ChromeTheme = new function(){
		
		var self = this;
		
		const LS_PREFIX = "chrometheme_";
		/*
		const BACKUP_PREFS = [
			"sd.background_color",
			"sd.background_color_enabled",
			"sd.background_url_type",
			"sd.background_url"
		];
		*/
		
		var themesConfig = null;
		
		// Events
		this.onThemeBgAvailable = new EventEmitter();
		
		function loadThemesConfig( callback ){
			
			fvdSpeedDial.Utils.getUrlContent( chrome.extension.getURL( "data/themes.json" ), function( content ){
															
				themesConfig = JSON.parse( content );
					
				if( callback ){
					callback();
				}
				
			} );
			
		}
		
		function _set( key, value ){
			localStorage[ LS_PREFIX + key ] = value;
		}
		
		function _get( key ){
			return localStorage[ LS_PREFIX + key ];
		}
		
		function restoreDefault(){
			
			var currentSdTheme = fvdSpeedDial.Prefs.get("sd.display_mode");

			var url = fvdSpeedDial.Prefs._themeDefaults[currentSdTheme]["sd.background_url"];
			
			function _end(){
				
				for( var k in fvdSpeedDial.Prefs._themeDefaults[currentSdTheme] ){
					fvdSpeedDial.Prefs.set( k, fvdSpeedDial.Prefs._themeDefaults[currentSdTheme][k] );			
				}
	
				_set( "last_offered_theme_id", "" );		
				_set( "current_bg_theme_id", "" );				
								
			}
			
			if( url ){
				
				fvdSpeedDial.Utils.imageUrlToDataUrl( url, function( du ){
					
					fvdSpeedDial.Storage.setMisc( "sd.background", du, function(){
						_end();
					});
																
				} );
				
			}
			else{
				_end();
			}
			
		}
		
		function currentThemeId( callback ){
			if( _get( "last_installed_theme" ) ){
				return callback( _get( "last_installed_theme" ) );
			}
			
			chrome.management.getAll( function( exts ){
				
				var themeId = null;
				
				for( var i = 0; i != exts.length; i++ ){
					
					var ext = exts[i];
					
					if( ext.type == "theme" ){
						themeId = ext.id;
					}
					
				}
				
				callback( themeId );
								
			} );
		}
		
		function getThemeData( themeId ){
			
			for( var i = 0; i != themesConfig.length; i++ ){
			 	if( themesConfig[i].id == themeId ){
					return themesConfig[i];
				}
			}
			
			return null;
			
		}
		
		this.useThemeBg = function( theme, callback ){
				
			fvdSpeedDial.Utils.imageUrlToDataUrl( theme.bgUrl, function( du ){
				
				self.doNotShowCurrentThemeOffer();
				
				fvdSpeedDial.Storage.setMisc( "sd.background", du, function(){
					fvdSpeedDial.Prefs.set( "sd.background_url", "<customFile>" );
					fvdSpeedDial.Prefs.set( "sd.background_url_type", "fill" );			
					
					_set( "current_bg_theme_id", theme.id );
					
					if( callback ){
						
						self.setPrefsForCurrentAppliedTheme();
												
						callback();
					}		
				});
															
			} );	
			
		}
		
		this.setPrefsForCurrentAppliedTheme = function(){
			
			var currentAppliedThemeId = _get( "current_bg_theme_id" );			
			
			if( currentAppliedThemeId ){
				var theme = self.getThemeData( currentAppliedThemeId );
				
				if( theme.prefs ){
					for( var k in theme.prefs ){
						fvdSpeedDial.Prefs.set( k, theme.prefs[k] );
					}
				}
			}
				
		}		
		
		this.getThemeData = function( themeId ){			
			var data = getThemeData( themeId );			

			data.bgUrl = this.getThemeBgUrl( themeId );
			data.thumbUrl = "http://fvdspeeddial.com/themes/" + themeId + "/thumb.png";
			
			return data;
		}
		
		this.getThemeBgUrl = function( themeId ){
			
			var data = getThemeData( themeId );
			
			var url = "http://fvdspeeddial.com/themes/" + themeId;
			
			var bgFilePath = "";
			
			if( typeof data.bgFilePath == "string" ){
				bgFilePath = data.bgFilePath;
			}
			else{
				
				var screenWidth = window.screen.width;
				
				var minDelta = 999999999;
				var minDeltaValue = "";
				var selectedWidth = 0;
				var backgrounds = data.bgFilePath;
				
				var widths = [];
				for( var w in backgrounds ){
					widths.push( parseInt(w) );
				}
				
				widths.sort();
								
				for( var w in backgrounds ){
					var delta = Math.abs( screenWidth - w );
					if( delta == 0 ){
						minDelta = delta;
						minDeltaValue = backgrounds[w];
						selectedWidth = w;
						break;
					}					
					
					if( delta < minDelta ){
						minDelta = delta;
						minDeltaValue = backgrounds[w];
						selectedWidth = w;
					}
				}
				
				if( minDelta != 0 && selectedWidth < screenWidth ){
					for( var i = 0; i != widths.length; i++ ){
						var w = widths[i];
																		
						if( w > selectedWidth ){							
							selectedWidth = w;
							minDeltaValue = backgrounds[w];
							break;
						}
					}
				}
				
				bgFilePath = minDeltaValue;
				
			}
			
			if( bgFilePath ){
				url += bgFilePath;
			}
			else{
				url += "/bg.jpg";
			}
			
			return url;
			
		}		
		
		this.needOfferThemeBg = function( callback ){
						
			currentThemeId( function( themeId ){
				
				if( !themeId ){
					return callback( false );
				}
				
				if( _get("last_offered_theme_id") != themeId ){
					
					if( getThemeData( themeId ) ){
						callback( true, themeId );						
					}
					else{
						callback( false );
					}
					
				}
				else{
					callback( false );
				}
			} );
			
		}
		
		// interface
		this.doNotShowCurrentThemeOffer = function(){
			
			currentThemeId(function( themeId ){
				_set("last_offered_theme_id", themeId);
			});
			
		}
		
		window.addEventListener( "load", function(){
			
			loadThemesConfig(function(){

				
			});
			
		} );
		
		
		
		chrome.management.onInstalled.addListener(function( ext ){
			
			if( ext.type == "theme" ){
				
				_set( "last_installed_theme", ext.id );
				
				loadThemesConfig(function(){
					
					_set("last_offered_theme_id", "");
					
					self.needOfferThemeBg( function( need, themeId ){
						
						if( need ){
							self.onThemeBgAvailable.callListeners( themeId );
						}
						
					} );
					
				});
				
			}
			
		});

		chrome.management.onUninstalled.addListener(function( id ){
			
			_set( "last_installed_theme", "" );
			
			if( _get( "current_bg_theme_id" ) == id ){
				restoreDefault();
			}
			
		});		
		
	}
	
}
else{
	
	fvdSpeedDial.ChromeTheme = chrome.extension.getBackgroundPage().fvdSpeedDial.ChromeTheme;
	
	fvdSpeedDial.ChromeThemeClient = new function(){
		
		var _theme = null;
		
		function displayChangeBgWindow( themeId ){
			
			var theme = fvdSpeedDial.ChromeTheme.getThemeData( themeId );			
			_theme = theme;
			
			fvdSpeedDial.SpeedDialMisc.showOptions( "offerAdjustBgToTheme", null, null, {
				top: 20,
				left: document.body.clientWidth/2 - document.getElementById("offerAdjustBgToTheme").offsetWidth / 2
			}, true );	

			document.getElementById( "themeBackgroundAdjustPreview" ).setAttribute("src", theme.thumbUrl);			
			
		}
		
		function _onThemeBgAvailable( themeId ){
			
			displayChangeBgWindow( themeId );
			
		}
		
		window.addEventListener( "unload", function(){
			
			fvdSpeedDial.ChromeTheme.onThemeBgAvailable.removeListener( _onThemeBgAvailable );
			
		}, false );
		
		window.addEventListener( "load", function(){
		
			fvdSpeedDial.ChromeTheme.onThemeBgAvailable.addListener( _onThemeBgAvailable );
			
			var btnApply = document.getElementById( "applyThemeBg" );
			var btnCancel = document.getElementById( "cancelThemeBg" );
			var thumb = document.getElementById( "themeBackgroundAdjustPreview" );
			var thumbContainer = document.getElementById("themeBackgroundAdjustPreviewContainer");
			
			// set events 
			btnApply.addEventListener( "click", function(){
				
				btnApply.setAttribute("hidden", true);
				btnCancel.setAttribute("hidden", true);
				thumbContainer.setAttribute( "loading", 1 );
				
				fvdSpeedDial.ChromeTheme.useThemeBg( _theme, function(){
					fvdSpeedDial.SpeedDial.refreshBackground();						
					fvdSpeedDial.SpeedDialMisc.hideOptions();
					
					setTimeout(function(){
						thumbContainer.removeAttribute( "loading" );
						btnApply.removeAttribute( "hidden" );
						btnCancel.removeAttribute( "hidden" );																
					}, 500);
				} );				
				
			} );
			
			btnCancel.addEventListener( "click", function(){
				
				fvdSpeedDial.ChromeTheme.doNotShowCurrentThemeOffer();
				fvdSpeedDial.SpeedDialMisc.hideOptions();
				
			} );
			
			fvdSpeedDial.ChromeTheme.needOfferThemeBg( function( need, themeId ){
				
				if( need ){
					
					setTimeout( function(){
						displayChangeBgWindow( themeId );
					}, 3000 );
					
				}
				
			} );
			
		} );
		
	}	
	
}
