// singletone

if (window == chrome.extension.getBackgroundPage()) {
	
	(function(){
		
		var Sync = function(){
			
			var fvdSynchronizerName = "EverSync";
			var fvdSynchronizerId = "iohcojnlgnfbmjfjfkbhahhmppcggdog";
			
			var self = this;
			
			var active = false;
			var port = null;

			var lastTransactionId = 0;
			
			var lastRequestId = 0;
			var pendingRequests = [];
			const PENDING_REQUESTS_TIMEOUT = 1000 * 5 * 60; // max life time of pending requests 
			
			// events			
			this.onSyncDataChanged = new EventEmitter();
			this.onActivityStateChanged = new EventEmitter();
			
			function setActivity( newActivity ){
				active = newActivity;
				
				self.onActivityStateChanged.callListeners();
			}
			
			function clearExpiredRequests(){
				
				var now = new Date().getTime();
				
				var ids = [];
				
				pendingRequests.forEach(function( request ){
					
					if( now - request.time >= PENDING_REQUESTS_TIMEOUT ){
						ids.push( request.id );
					}
					
				});				
				
				ids.forEach( function( id ){
					
					removePendingRequest( id );
					
				} );				
				
			}
			
			function removePendingRequest( id ){
				
				var index = -1;
				
				for( var i = 0; i != pendingRequests.length; i++ ){
					if( pendingRequests[i].id == id ){
						index = i;
						break;
					}					
				}
				
				if( index != -1 ){
					pendingRequests.splice( index, 1 );
				}
				
			}
			
			function requestWithCallback( data, callback ){
				
			  	if( !port ){
					return callback( null );
				}
										
				lastRequestId++;
				data.requestId = lastRequestId;
				
				pendingRequests.push({
					id: lastRequestId,
					callback: callback,
					time: new Date().getTime()
				});
				
				try{
					port.postMessage( data );					
				}
				catch( ex ){

				}				
				
			}
			
			function callRequestResponse( requestId, data ){
				
				pendingRequests.forEach( function( request ){
					
					if( request.id == requestId ){
						request.callback( data );
					}
					
				} );
				
				removePendingRequest( requestId );
				
			}
			
			this.isActive = function(){
				
				return active;
				
			};
			
			this.hasDataToSync = function( callback ){
				requestWithCallback( {
					action: "hasToSyncData"
				}, function( response ){
					
					callback( response && response.has );
					
				} );
			};		
			
			this.groupSyncChanged = function( groupId, callback ){				
				
				// need sync group as new and all it dials.
				
				fvdSpeedDial.Storage.getGroup( groupId, function( group ){

					if( group.sync == 1 ){
						
						fvdSpeedDial.Utils.Async.chain([
							function( chainCallback ){
								
								// remove data from sync
								fvdSpeedDial.Sync.removeSyncData( ["deleteGroups"], group.global_id, {}, function(){
									chainCallback();
								} );
								
							},
							
							function( chainCallback ){
								
								fvdSpeedDial.Storage.listDialsIdsByGroup( groupId, function( dials ){
									
									fvdSpeedDial.Utils.Async.arrayProcess( dials, function( dial, arrayProcessCallback ){
										
										fvdSpeedDial.Sync.removeSyncData( ["deleteDials"], dial.global_id, {}, arrayProcessCallback );	
										
									}, function(){
										chainCallback();
									} );
									
								} );
									
							},
							
							function( chainCallback ){
								
								// add data to sync		
								
								// need to sync all groups
								
								fvdSpeedDial.Storage.groupsRawList( {}, function( groups ){
									
									fvdSpeedDial.Utils.Async.arrayProcess( groups, function( group, arrayProcessCallback ){
										
										if( group.sync == 1 ){
											fvdSpeedDial.Sync.addDataToSync( ["groups", "newGroups"], group.global_id, {syncAnyWay: true}, arrayProcessCallback );									
										}
										else{
											arrayProcessCallback();
										}
										
									}, function(){
										chainCallback();
									} );
											
								} );
								
							},
							
							function( chainCallback ){
								fvdSpeedDial.Sync.addDataToSync( ["specialActions"], "merge_group:" + groupId + ":" + group.global_id, {}, chainCallback );
							},
							
							function(){
									
								if( callback ){
									callback();
								}	
							}							
						]);
						
												
						/*
						
						fvdSpeedDial.Storage.listDialsIdsByGroup( groupId, function( dials ){
							
							console.log(dials);
							
							dials.forEach( function( dial ){
								fvdSpeedDial.Sync.addDataToSync( ["dials", "newDials"], dial.global_id );	
							} );
							
						} );
						
						*/
						
					}
					else if( group.sync == 0 ){
						
						fvdSpeedDial.Utils.Async.chain( [
							
							function( chainCallback ){
								// remove data from sync
								fvdSpeedDial.Sync.removeSyncData( ["groups", "newGroups"], group.global_id, {}, chainCallback );
							},
							
							function( chainCallback ){
								fvdSpeedDial.Sync.removeSyncData( ["specialActions"], "merge_group:" + groupId + ":" + group.global_id, {}, chainCallback );
							},
							
							function( chainCallback ){
								// add data to sync						
								fvdSpeedDial.Sync.addDataToSync( ["deleteGroups"], group.global_id, {syncAnyWay: true}, chainCallback );	
							},
							
							function( chainCallback ){
								
								fvdSpeedDial.Storage.listDialsIdsByGroup( groupId, function( dials ){
									
									fvdSpeedDial.Utils.Async.arrayProcess( dials, function( dial, arrayProcessCallback ){
										
										fvdSpeedDial.Sync.addDataToSync( ["deleteDials"], dial.global_id, {syncAnyWay: true}, arrayProcessCallback );	
										
									}, chainCallback );
														
								} );
								
							},
							
							function(){
								
								chrome.extension.sendRequest({
									message: "displayNoSyncGroupAlert",
									needActiveTab: true
								});								
								
								if( callback ){
									callback();
								}
										
							}
							
						] );
						
						
					}
					
				} );
							
				
			};
			
			this.syncAddonOptionsUrl = function( callback ){
				getSynchronizerId( function( id ){
					var url = null;
					if( id ){
						url = "chrome-extension://"+id+"/options.html";
					}	
					callback( url );				
				} );
			};
			
			this.syncAddonExists = function( callback ){
				getSynchronizerId( function( id ){
					var exists = false;
					if( id ){
						exists = true;						
					}	
					callback( exists );				
				} );
			};
			
			this.removeSyncData = function( category, _data, params, callback ){
				
				var data = _data;
				
				params = params || {};
				
				if( !active ){
				 	if( callback ){
						callback();
					}					
					return;
				}
				
				if( typeof category == "string" ){
					category = [category];
				}
				
				requestWithCallback( {
					action: "removeSyncData",
					data: data,
					category: category
				}, function(){
					
					if( callback ){
						callback();
					}
					
				} );
				
			};
			
			this.importFinished = function(){
				
				port.postMessage( {
					action: "importFinished"
				} );
			
			};
			
			this.addDataToSync = function( category, _data, params, callback ){
				
				params = params || {};

				
				if( !active ){
				 	if( callback ){
						callback();
					}					
					return;
				}
				
				
				var data = null;
				
				if( typeof category == "string" ){
					category = [category];
				}
				
				fvdSpeedDial.Utils.Async.chain( [
					function( chainCallback ){
						if( params.translate ){
							
							if( params.translate == "group" ){
								fvdSpeedDial.Storage.groupGlobalId( _data, function( globalId ){
									data = globalId;
									
									chainCallback();
								} );
							}
							else if( params.translate == "dial" ){
								fvdSpeedDial.Storage.dialGlobalId( _data, function( globalId ){
									data = globalId;
									
									chainCallback();
								} );								
							}
							else{
								console.log( "Undefined translate", params.translate );
							}						
							
						}						
						else{
							data = _data;
							chainCallback();
						}						
					},
					
					function( chainCallback ){
								
						if( params.syncAnyWay ){
							chainCallback();
							return;
						}		
						
						/*						
						if( category.indexOf( "groups" ) != -1 || category.indexOf( "deleteGroups" ) != -1 ){
							
							fvdSpeedDial.Storage.groupCanSync( data, function( can ){
									
								if( can ){
									chainCallback();
								}
								else{
									if( callback ){
										callback();	
									}
								}
								
							} );
								
						}
						else if( category.indexOf( "dials" ) != -1 || category.indexOf( "deleteDials" ) != -1 ){
							
							console.log( "Check dials" );
							
							fvdSpeedDial.Storage.dialCanSync( data, function( can ){
								
								console.log( "D", data, can );
								
								if( can ){
									chainCallback();
								}
								else{
									if( callback ){
										callback();	
									}
								}
								
							} );
						}
						else{
							chainCallback();	
						}
						*/
						
						chainCallback();
						
					},
					
					function(){
						
						requestWithCallback( {
							action: "addDataToSync",
							data: data,
							category: category
						}, function(){
							
							if( callback ){
								callback();
							}
							
						} );
						

						
					}
				] );
	

					
			};
			
			this.startSync = function( type, callback ){
				
				type = type || "main";
				
				requestWithCallback( {
					action: "startSync",
					type: type
				}, function( result ){
					
					if( callback ){
						callback( result.state );
					}
					
				} );				
				
			};

			
			function clearGroupsAndDials( callback ){
				
				fvdSpeedDial.Utils.Async.chain( [
					function( chainCallback ){
						fvdSpeedDial.Storage.clearDials( function(){
							chainCallback();
						}, "(SELECT `groups`.`sync` FROM `groups` WHERE `groups`.`rowid` = `dials`.`group_id`) = 1" );
					},
					function( chainCallback ){						
						fvdSpeedDial.Storage.clearGroups( function(){
							chainCallback();
						}, "`sync` = 1" );
						
					},
					function(){
						callback();
					}
				] );
				
				
				
			}
			
			function getStorage( message, callback ){
				
				callback( fvdSpeedDial.Storage );			
				
			}
			
			// transactions
			
			function createTransaction( callback ){
				
				lastTransactionId++;
				var id = lastTransactionId;
				
				fvdSpeedDial.Storage.backupTables( ["dials", "groups"], id, function(){
					callback( id );
				} );
				
			}
			
			function commitTransaction( id, callback ){		
					
				fvdSpeedDial.Storage.removeBackup( id, callback );	
								
			}
			
			function rollbackTransaction( id, callback ){
				
				fvdSpeedDial.Storage.restoreBackup( id, callback );
				
			}

			function processPortMessage( message ){
					
				console.log( "REQUESTED ", message.action );
								
				switch( message.action ){
					// transactions
					
					case "startTransaction":
					
						createTransaction(function( id ){
							
							console.log( "Created trans", id );
							
							sendResponse( message.requestId, {
								transId:id
							} );
							
						});
					
					break;
					
					case "rollbackTransaction":
						
						rollbackTransaction( message.transId, function(){
							sendResponse( message.requestId, {} );
						});
						
					break;
					
					case "commitTransaction":
					
						commitTransaction( message.transId, function(){
							sendResponse( message.requestId, {} );
						});
						
					break;
					
					case "syncStartNotification":
										
						chrome.extension.sendMessage( {action:"syncStartNotification"} );
						
					break;
					
					case "syncEndNotification":
						
						chrome.extension.sendMessage( {action:"syncEndNotification"} );
						
					break;
					
					case "clearGroupsAndDials":
					
						clearGroupsAndDials( function(){
							sendResponse( message.requestId, {} );
						} );
					
					break;
					
					case "toSyncDataChanged":

						self.onSyncDataChanged.callListeners( message.toSyncData );
					
					break;
					
					case "restoreBackup":
					
						getStorage( message, function( s ){
							
							s.restoreTablesData( message.data, function(){
								sendResponse( message.requestId, {
									
								} );								
							}, function( current, max ){
								
								port.postMessage( {
									action: "restoreBackupProgress",
									current: current,
									max: max
								} );
								
							} );
							
						} );
					
					break;
					
					case "queryGroups":
						
						getStorage( message, function( s ){
							
							if( !message.ignoreNoSync ){
								var whereSync = "`groups`.`sync` = 1";
	
								if( message.where ){
									message.where.push( whereSync );
								}
								else{
									message.where = [whereSync];
								}
							}
							
							s.groupsRawList( message, function( list ){
								
								sendResponse( message.requestId, {
									list: list
								} );
								
							} );
						} );

						
					break;
					
					case "queryDials":
						
						getStorage( message, function( s ){
							
							if( !message.ignoreNoSync ){
								
								var whereSync = "(SELECT `sync` FROM `groups` WHERE `groups`.`rowid` = `dials`.`group_id`) = 1";
	
								if( message.where ){
									message.where.push( whereSync );
								}
								else{
									message.where = [whereSync];
								}
								
							}
							
							s.dialsRawList( message, function( list ){
								
								sendResponse( message.requestId, {
									list: list
								} );
								
							} );
						} );
						

						
					break;
					
					case "mergeUpdateCollisedGroup":
						
						getStorage( message, function( s ){
							
							s.groupUpdate( message.clientGroup.rowid, {							
								global_id: message.serverGroup.global_id
							}, function(){
								sendResponse( message.requestId );
							} );
							
						} );

						
					break;
					
					case "mergeUpdateCollisedDial":
					
						getStorage( message, function( s ){							
							
							s.updateDial( message.clientDial.rowid, {							
								global_id: message.serverDial.global_id
							}, function(){
								
								fvdSpeedDial.Storage.dialCanSync( message.serverDial.global_id, function( can ){
									
									if( can ){
										s.syncSaveDial( message.serverDial, function( saveInfo ){
											sendResponse( message.requestId, {
												saveInfo: saveInfo
											} );
										} );	
									}	
									else{
										sendResponse( message.requestId, {
											saveInfo: null
										} );
									}								
	
								} );
								
							} );
							
						});
					
					break;					
					
					case "saveGroup":
					
						getStorage( message, function( s ){
							
							fvdSpeedDial.Storage.groupCanSync( message.group.global_id, function( can ){
								
							 	if( can ){
									s.syncSaveGroup( message.group, function(){							
										sendResponse( message.requestId );							
									} );
								}
								else{
									sendResponse( message.requestId );	
								}
								
							} );
							
						});

						
					break;		
					
					case "moveDial":
						
						getStorage( message, function( s ){
							
							s.moveDial( message.id, message.groupId, function(){
								sendResponse( message.requestId );
							} );
							
						});

						
					break;
					
					case "saveDial":
					
						getStorage( message, function( s ){
							
							console.log("OBtained message", message);
							
							s.syncSaveDial( message.dial, function( saveInfo ){							
								sendResponse( message.requestId, {
									saveInfo: saveInfo
								} );							
							} );
							

						});
					
					break;	
					
					case "updateMass":
					
						getStorage( message, function( s ){
							
							console.log("OBtained message", message);
							
							s.syncUpdateMass( message.globalIds, message.data, function(){	
								if( message.requestId ){
									sendResponse( message.requestId, {
										
									} );		
								}
							} );							

						});
					
					break;
					
					case "removeGroupsNotInList":
					
						var notRemoveGroups = message.groupsIds;
						
						getStorage( message, function( s ){
							
							s.syncRemoveGroups( notRemoveGroups, function( removed ){
								sendResponse(message.requestId, {
									removed: removed
								});
							} );		
							
						});
											
					break;		
					
					case "removeDialsNotInList":
					
						var notRemoveDials = message.dialsIds;
						
						getStorage( message, function( s ){
							
							s.syncRemoveDials( notRemoveDials, function( removeInfo ){
								sendResponse(message.requestId, {
									removeInfo: removeInfo
								});
							} );
							
						});
						

					
					break;	
					
					case "fixGroupsPositions":
					
						getStorage( message, function( s ){
							s.syncFixGroupsPositions( function(){
								
								sendResponse(message.requestId);
								
							} );					
						});					
					

					break;
					
					case "fixDialsPositions":
					
						getStorage( message, function( s ){
							s.syncFixDialsPositions( message.groupId, function(){
								
								sendResponse(message.requestId);
								
							} );
						});					
					

					
					break;
					
					case "rebuild":
					
						fvdSpeedDial.Observer.sendToAllExcludeCurrent( {
							message: "forceRebuild",
							resetActiveGroup: true
						} );
						
						// need to refresh default group id
						
						var activeGroupId = fvdSpeedDial.Prefs.get( "sd.default_group" );
						
						console.log("Active group id", activeGroupId);
						
						if( activeGroupId && activeGroupId > 0 ){
							
							fvdSpeedDial.Storage.groupExistsById( activeGroupId, function( exists ){
								
								console.log( "Exists? ", exists );
								
								if( !exists ){
									fvdSpeedDial.Storage.resetDefaultGroupId();									
								}								
							} );							
							
						}
												
						fvdSpeedDial.ContextMenu.rebuild();	
					
					break;
					
					case "syncAllGroupContents":
					
						fvdSpeedDial.Storage.syncGetGroupId( message.groupGlobalId, function( id ){
							
							if( id ){
								fvdSpeedDial.Sync.groupSyncChanged( id, function(){
									
									sendResponse(message.requestId);
									
								} );
							}
							else{
								sendResponse(message.requestId);
							}
							
						} );	

					break;
					
					case "addGroupDialsToSync":
					
						fvdSpeedDial.Utils.Async.chain( [
							function( chainCallback ){
								
								fvdSpeedDial.Sync.addDataToSync( ["groups"], message.groupGlobalId, {}, chainCallback );
								
							},
							
							function(){
								
								fvdSpeedDial.Storage.syncGetGroupId( message.groupGlobalId, function( groupId ){
									
									fvdSpeedDial.Storage.listDialsIdsByGroup( groupId, function( dials ){
										
										fvdSpeedDial.Utils.Async.arrayProcess( dials, function( dial, arrayProcessCallback ){
											
											fvdSpeedDial.Sync.addDataToSync( ["dials", "newDials"], dial.global_id, {}, arrayProcessCallback );	
											
										}, function(){
											
											sendResponse(message.requestId);
											
										} );
															
									} );								
									
								} );	
								
							}
						] );


					break;
					
					case "setGroupSync":
					
						fvdSpeedDial.Storage.syncGetGroupId( message.global_id, function( groupId ){
							
							if( !groupId ){
								sendResponse(message.requestId);					
							}
							else{
								fvdSpeedDial.Storage.groupUpdate( groupId, {
									sync: message.value
								}, function(){
									
									sendResponse(message.requestId);					
									
								} );								
							}
							
							
						} );
					

					
					break;
					
					case "dialCanSync":
					
						fvdSpeedDial.Storage.dialCanSync( message.global_id, function( can ){
							
							sendResponse(message.requestId, {can: can});					
							
						} );
					
					break;
					
					case "countItems":
					
						var result = {};
						
						fvdSpeedDial.Utils.Async.chain([
							
							function( chainCallback ){
								
								fvdSpeedDial.Storage.countDials( function( countDials ){
									result.dials = countDials;
									chainCallback();
								} );
								
							},
							function( chainCallback ){
								fvdSpeedDial.Storage.groupsCount( function( countGroups ){									
									result.groups = countGroups;
									chainCallback();									
								} );
							},
							function(  ){
								sendResponse(message.requestId, result);	
							}
							
						]);
					
						
						
					break;
					
					case "_response":
					
						callRequestResponse( message.requestId, message );
					
					break;
						
					
				}
				
			}
			
			function sendResponse( requestId, message ){
				
				message = message || {};
				
				message.action = "_response";
				message.requestId = requestId;
				
				port.postMessage( message );
				
			}
			
			function getSynchronizerId( callback ){
				
				chrome.management.getAll(function( results ){
					
					var id = null;
					
					results.forEach(function( extension ){
						
						if( extension.name == fvdSynchronizerName && extension.enabled || extension.id == fvdSynchronizerId ){
							id = extension.id;
						}
						
					});
					
					callback( id );
					
				});
				
			}		
			
			chrome.extension.onMessageExternal.addListener(function( message, sender, callback ){

				switch( message.action ){
					
					// say hello used to check from another extensions(like as fvd synchronizer) active fvdSpeedDial or not. If it active it says hello in response
					// this action used in fvdSynchronizer to wait fvdSpeedDial to be actived after Enabling or Installing by user and establish connection with it.
					case "sayHello":
					
						callback( "hello" );
					
					break;
					
				}
				
			});	

			chrome.extension.onConnectExternal.addListener(function( _p, sender ) {
							
				console.log( "External connection" );
				
				if( !_p.sender ){
					console.log( "Sender not specified" );
					return;
				}

				
				port = _p;
				
				port.onMessage.addListener( processPortMessage );
				port.onDisconnect.addListener( function(){
					setActivity(false);
				} );
				
				if (fvdSpeedDial.PowerOff.isHidden()) {
					// do not send activate message to synchronizer if speed dial is locked
					return;
				}
				
				port.postMessage( {
					"action": "activate"
				} );
				
				console.log( "Connection with FVD Sync activated" );
				
			 	if( !_b( fvdSpeedDial.Prefs.get("sd.synced_after_install") ) ){
					fvdSpeedDial.Prefs.set("sd.synced_after_install", true);
					port.postMessage( {
						"action": "needInitialSync"
					} );
				}
				
				
				setActivity(true);
				
			});
			
			setInterval( function(){
				clearExpiredRequests();
			}, 1000 * 60 * 5 );
			
			window.addEventListener( "load", function(){
				
				fvdSpeedDial.PowerOff.onHiddenChange.addListener( function(){
					
					if( port ){
						
						try{
							
							if( fvdSpeedDial.PowerOff.isHidden() ){
								port.postMessage( {
									"action": "deactivate"
								} );
							}
							else{
								port.postMessage( {
									"action": "activate"
								} );								
							}
							
						}
						catch( ex ){
							
						}
						
					}
					
				} );				
				
			}, false );
						
		};
		
		this.Sync = new Sync();
		
	}).apply( fvdSpeedDial );
	
}
else{
	fvdSpeedDial.Sync = chrome.extension.getBackgroundPage().fvdSpeedDial.Sync;
}
